@@ -9,7 +9,8 @@ def test_find_subset():
 def test_sequence_matcher():
     assert dpss.sequence_matcher(
         [1980, 2980, 3500, 4000, 1050],
-        [1950, 2900, 30, 80, 3300, 200, 3980, 1050, 20], 3, 3, 1000) == \
+        [1950, 2900, 30, 80, 3300, 200, 3980, 1050, 20], 3, 3, 
+        1000, True, True) == \
         [[([1050], [1050]),
           ([1980], [30, 1950]),
             ([2980], [80, 2900]),
